// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assert.hpp"

namespace Dawn {
  class StateOwner;

  template<typename...A>
  class StateEvent {
    private:
      std::vector<std::function<void(A...)>> listeners;

    public:
      /**
       * Invokes the event and emits to all of the listeners.
       * 
       * @param args Arguments for this event to pass to the listeners.
       */
      void invoke(A... args) {
        auto itListeners = this->listeners.begin();
        while(itListeners != this->listeners.end()) {
          (*itListeners)(args...);
          ++itListeners;
        }
      }

      friend class StateOwner;
  };
}