// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assert.hpp"

namespace Dawn {
  class StateOwner;

  template<class V>
  class StateProperty {
    private:
      StateOwner *owner;
      V value;
      
      /**
       * Method that is invoked every time that the value of this state property
       * is updated.
       * 
       * @param val Value that is to be used for this property.
       */
      void setInternal(V val) {
        if(val == this->value) return;// TODO: can I omit this? kinda bad tbh.
        assertNotNull(this->owner);
        auto old = this->value;
        this->value = val;
        this->owner->_statePropertyUpdated(this, val, old);
      }

    public:
      const StateProperty& operator += (const V &value) {
        this->setInternal(this->value + value);
        return *this;
      }

      const bool_t operator != (const V &value) {
        return value != this->value;
      }

      const bool_t operator == (const V &value) {
        return value == this->value;
      }

      const StateProperty& operator = (const V &val) {
        this->setInternal(val);
        return *this;
      }

      operator V() const {
        return this->value;
      }

      /**
       * Destructor for StateProperty.
       */
      ~StateProperty() {
        this->owner->_statePropertyDestroyed(this);
      }

      friend class StateOwner;
  };
}